TRANSLATION = {
    "Delete": {
        "source": [
            "Furious.QtFramework.QtWidgets",
            "Furious.Widget.UserServersQTableWidget",
            "Furious.Widget.UserSubsQTableWidget",
            "Furious.Window.UserSubsWindow",
            "Furious.Widget.XrayAssetViewerQListWidget"
        ],
        "RU": "Удалить",
        "ZH": "删除",
        "isReviewed": "True"
    },
    "Delete these items?": {
        "source": [
            "Furious.QtFramework.QtWidgets"
        ],
        "RU": "Удалить эти элементы",
        "ZH": "删除这些项目？",
        "isReviewed": "True"
    },
    "Delete this item?": {
        "source": [
            "Furious.QtFramework.QtWidgets"
        ],
        "RU": "Удалить этот элемент",
        "ZH": "删除此项目？",
        "isReviewed": "True"
    },
    "New changes will take effect next time": {
        "source": [
            "Furious.QtFramework.QtWidgets"
        ],
        "RU": "Новые изменения вступят в силу в следующий раз",
        "ZH": "新的更改将于下次生效",
        "isReviewed": "True"
    },
    "New version available": {
        "source": [
            "Furious.QtFramework.UpdatesManager"
        ],
        "RU": "Доступна новая версия",
        "ZH": "新版本可用",
        "isReviewed": "True"
    },
    "Check for updates failed": {
        "source": [
            "Furious.QtFramework.UpdatesManager"
        ],
        "RU": "Не удалось проверить наличие обновлений",
        "ZH": "检查更新失败",
        "isReviewed": "True"
    },
    "Go to download page?": {
        "source": [
            "Furious.QtFramework.UpdatesManager"
        ],
        "RU": "Перейти на страницу загрузки?",
        "ZH": "前往下载页面？",
        "isReviewed": "True"
    },
    "Furious is already the latest version": {
        "source": [
            "Furious.QtFramework.UpdatesManager"
        ],
        "RU": "Furious - это уже последняя версия",
        "ZH": "Furious已是最新版本",
        "isReviewed": "True"
    },
    "Invalid data": {
        "source": [
            "Furious.TrayActions.Import"
        ],
        "RU": "Неверные данные",
        "ZH": "无效的数据",
        "isReviewed": "True"
    },
    "Invalid data. The content of the clipboard is:": {
        "source": [
            "Furious.TrayActions.Import"
        ],
        "RU": "Неверные данные. Содержимое буфера обмена является:",
        "ZH": "无效的数据。剪贴板内容为",
        "isReviewed": "True"
    },
    "Import": {
        "source": [
            "Furious.TrayActions.Import",
            "Furious.Widget.XrayAssetViewerQListWidget"
        ],
        "RU": "Импорт",
        "ZH": "导入",
        "isReviewed": "True"
    },
    "Import share link success": {
        "source": [
            "Furious.TrayActions.Import"
        ],
        "RU": "Успешный импорт общей ссылки",
        "ZH": "导入分享链接成功",
        "isReviewed": "True"
    },
    "Imported to row": {
        "source": [
            "Furious.TrayActions.Import"
        ],
        "RU": "Импорт в строку",
        "ZH": "导入至行",
        "isReviewed": "True"
    },
    "Import success": {
        "source": [
            "Furious.TrayActions.Import"
        ],
        "RU": "Успешный импорт",
        "ZH": "导入成功",
        "isReviewed": "True"
    },
    "Import From File...": {
        "source": [
            "Furious.TrayActions.Import",
            "Furious.Window.XrayAssetViewerWindow"
        ],
        "RU": "Импорт из файла...",
        "ZH": "从文件导入...",
        "isReviewed": "True"
    },
    "Import File": {
        "source": [
            "Furious.TrayActions.Import",
            "Furious.Window.XrayAssetViewerWindow"
        ],
        "RU": "Импорт файла",
        "ZH": "导入文件",
        "isReviewed": "True"
    },
    "Text files (*.json);;All files (*)": {
        "source": [
            "Furious.TrayActions.Import",
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "Текст файлов (*.json);;Остальные файлы (*)",
        "ZH": "文本文件 (*.json);;所有文件 (*)",
        "isReviewed": "True"
    },
    "Error opening file": {
        "source": [
            "Furious.TrayActions.Import"
        ],
        "RU": "Ошибка открытия файла",
        "ZH": "打开文件出错",
        "isReviewed": "True"
    },
    "Invalid configuration file": {
        "source": [
            "Furious.TrayActions.Import"
        ],
        "RU": "Недопустимый конфигурационный файл",
        "ZH": "无效的配置文件",
        "isReviewed": "True"
    },
    "Import Share Link From Clipboard": {
        "source": [
            "Furious.TrayActions.Import"
        ],
        "RU": "Импортировать ссылку с буфера обмена",
        "ZH": "从剪贴板导入分享链接",
        "isReviewed": "True"
    },
    "Import JSON Configuration From Clipboard": {
        "source": [
            "Furious.TrayActions.Import"
        ],
        "RU": "Импортировать Json конфигурацию с буфера обмена",
        "ZH": "从剪贴板导入JSON配置",
        "isReviewed": "True"
    },
    "Set Indent": {
        "source": [
            "Furious.Widget.IndentSpinBox"
        ],
        "RU": "Установить отступ",
        "ZH": "设置缩进",
        "isReviewed": "True"
    },
    "Indent:": {
        "source": [
            "Furious.Widget.IndentSpinBox"
        ],
        "RU": "Отступ",
        "ZH": "缩进：",
        "isReviewed": "True"
    },
    "Cancel": {
        "source": [
            "Furious.QtFramework.GuiEditorXXX",
            "Furious.Widget.IndentSpinBox",
            "Furious.Window.TextEditorWindow",
            "Furious.Window.UserSubsWindow"
        ],
        "RU": "Отменить",
        "ZH": "取消",
        "isReviewed": "True"
    },
    "Save Changes": {
        "source": [
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "Сохранить изменения",
        "ZH": "保存更改",
        "isReviewed": "True"
    },
    "The content has been modified. Save changes?": {
        "source": [
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "Содержимое было изменено. Сохранить изменения?",
        "ZH": "内容已修改。保存更改？",
        "isReviewed": "True"
    },
    "Save": {
        "source": [
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "Сохранить",
        "ZH": "保存",
        "isReviewed": "True"
    },
    "Discard": {
        "source": [
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "Отбросить",
        "ZH": "丢弃",
        "isReviewed": "True"
    },
    "Please check if the configuration is in valid JSON format": {
        "source": [
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "В конфигурации недопустимый формат JSON, проверьте",
        "ZH": "请检查配置是否为有效的JSON格式",
        "isReviewed": "True"
    },
    "Save As...": {
        "source": [
            "Furious.Window.TextEditorWindow",
            "Furious.Window.LogViewerWindow"
        ],
        "RU": "Сохранить как...",
        "ZH": "另存为...",
        "isReviewed": "True"
    },
    "Exit": {
        "source": [
            "Furious.Window.TextEditorWindow",
            "Furious.Window.LogViewerWindow",
            "Furious.Window.XrayAssetViewerWindow",
            "Furious.TrayActions.Exit"
        ],
        "RU": "Выход",
        "ZH": "退出",
        "isReviewed": "True"
    },
    "File": {
        "source": [
            "Furious.Window.TextEditorWindow",
            "Furious.Window.LogViewerWindow",
            "Furious.Window.XrayAssetViewerWindow"
        ],
        "RU": "Файл",
        "ZH": "文件",
        "isReviewed": "True"
    },
    "Undo": {
        "source": [
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "Отменить",
        "ZH": "撤销",
        "isReviewed": "True"
    },
    "Redo": {
        "source": [
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "Переделать",
        "ZH": "重做",
        "isReviewed": "True"
    },
    "Cut": {
        "source": [
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "Вырезать",
        "ZH": "剪切",
        "isReviewed": "True"
    },
    "Copy": {
        "source": [
            "Furious.Window.TextEditorWindow",
            "Furious.Window.LogViewerWindow"
        ],
        "RU": "Копировать",
        "ZH": "复制",
        "isReviewed": "True"
    },
    "Paste": {
        "source": [
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "Вставить",
        "ZH": "粘贴",
        "isReviewed": "True"
    },
    "Select All": {
        "source": [
            "Furious.Window.TextEditorWindow",
            "Furious.Widget.UserServersQTableWidget",
            "Furious.Window.LogViewerWindow"
        ],
        "RU": "Выбрать все",
        "ZH": "全选",
        "isReviewed": "True"
    },
    "Indent...": {
        "source": [
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "Отступ",
        "ZH": "缩进...",
        "isReviewed": "True"
    },
    "Edit": {
        "source": [
            "Furious.Window.TextEditorWindow",
            "Furious.Window.LogViewerWindow"
        ],
        "RU": "Редактировать",
        "ZH": "编辑",
        "isReviewed": "True"
    },
    "Zoom In": {
        "source": [
            "Furious.Window.TextEditorWindow",
            "Furious.Window.LogViewerWindow"
        ],
        "RU": "Увеличить маштаб",
        "ZH": "变大",
        "isReviewed": "True"
    },
    "Zoom Out": {
        "source": [
            "Furious.Window.TextEditorWindow",
            "Furious.Window.LogViewerWindow"
        ],
        "RU": "Уменьшить маштаб",
        "ZH": "变小",
        "isReviewed": "True"
    },
    "View": {
        "source": [
            "Furious.Window.TextEditorWindow",
            "Furious.Window.LogViewerWindow"
        ],
        "RU": "Вид",
        "ZH": "显示",
        "isReviewed": "True"
    },
    "Error saving configuration": {
        "source": [
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "Конфигурация с сохранением ошибок",
        "ZH": "保存配置出错",
        "isReviewed": "True"
    },
    "Save File": {
        "source": [
            "Furious.Window.TextEditorWindow",
            "Furious.Window.LogViewerWindow"
        ],
        "RU": "Сохранить файл",
        "ZH": "保存文件",
        "isReviewed": "True"
    },
    "Error Saving File": {
        "source": [
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "Ошибка сохранения файла",
        "ZH": "保存文件出错",
        "isReviewed": "True"
    },
    "Invalid server configuration": {
        "source": [
            "Furious.Window.TextEditorWindow",
            "Furious.TrayActions.Connect"
        ],
        "RU": "Неверная конфигурация сервера",
        "ZH": "无效的服务器配置",
        "isReviewed": "True"
    },
    "Error setting indent": {
        "source": [
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "Ошибка при установке отступов внутрь и наружу",
        "ZH": "设置缩进出错",
        "isReviewed": "True"
    },
    "Remark": {
        "source": [
            "Furious.Widget.GuiHysteria1",
            "Furious.Widget.GuiHysteria2",
            "Furious.Widget.UserServersQTableWidget",
            "Furious.Widget.UserSubsQTableWidget"
        ],
        "RU": "Прозвище",
        "ZH": "别名",
        "isReviewed": "True"
    },
    "Protocol": {
        "source": [
            "Furious.Widget.GuiHysteria1",
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Протокол",
        "ZH": "协议",
        "isReviewed": "True"
    },
    "Address": {
        "source": [
            "Furious.Widget.GuiShadowsocks",
            "Furious.Widget.GuiTrojan",
            "Furious.Widget.GuiVLESS",
            "Furious.Widget.GuiVMess",
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Адрес",
        "ZH": "地址",
        "isReviewed": "True"
    },
    "Port": {
        "source": [
            "Furious.Widget.GuiShadowsocks",
            "Furious.Widget.GuiTrojan",
            "Furious.Widget.GuiVLESS",
            "Furious.Widget.GuiVMess",
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Порт",
        "ZH": "端口",
        "isReviewed": "True"
    },
    "Transport": {
        "source": [
            "Furious.Widget.GuiVTransport",
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Способ передачи",
        "ZH": "传输方式",
        "isReviewed": "True"
    },
    "TLS": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "TLS",
        "ZH": "TLS",
        "isReviewed": "True"
    },
    "Subscription": {
        "source": [
            "Furious.Widget.UserServersQTableWidget",
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Подписка",
        "ZH": "订阅",
        "isReviewed": "True"
    },
    "Latency": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Задержка",
        "ZH": "延迟",
        "isReviewed": "True"
    },
    "Speed": {
        "source": [
            "Furious.Widget.GuiHysteria1",
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Скорость",
        "ZH": "速度",
        "isReviewed": "True"
    },
    "Edit Configuration...": {
        "source": [
            "Furious.TrayActions.EditConfiguration"
        ],
        "RU": "Отредактируйте конфигурацию...",
        "ZH": "编辑配置...",
        "isReviewed": "True"
    },
    "Move Up": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Двигать вверх",
        "ZH": "上移",
        "isReviewed": "True"
    },
    "Move Down": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Двигать в низ",
        "ZH": "下移",
        "isReviewed": "True"
    },
    "Duplicate": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Дубликат",
        "ZH": "克隆",
        "isReviewed": "True"
    },
    "Scroll To Activated Server": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Перейдите к активированному серверу",
        "ZH": "滚动到激活的服务器",
        "isReviewed": "True"
    },
    "Test Ping Latency": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Задержка тестового пинга",
        "ZH": "测试Ping延迟",
        "isReviewed": "True"
    },
    "Test Download Speed": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Проверьте скорость загрузки",
        "ZH": "测试下行速度",
        "isReviewed": "True"
    },
    "Clear Test Results": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Четкие результаты тестирования",
        "ZH": "清除测试结果",
        "isReviewed": "True"
    },
    "New Empty Configuration": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Создайте новую пустую конфигурацию",
        "ZH": "新建空白配置",
        "isReviewed": "True"
    },
    "Export Share Link To Clipboard": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Копировать ссылку в буфер обмена",
        "ZH": "导出分享链接至剪贴板",
        "isReviewed": "True"
    },
    "Export As QR Code": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Экспорт в виде QR-кода",
        "ZH": "导出为二维码",
        "isReviewed": "True"
    },
    "Export JSON Configuration To Clipboard": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Копировать конфигурацию JSON в буфер обмена",
        "ZH": "导出JSON配置至剪贴板",
        "isReviewed": "True"
    },
    "Connecting": {
        "source": [
            "Furious.Widget.UserServersQTableWidget",
            "Furious.TrayActions.Connect"
        ],
        "RU": "Соединение",
        "ZH": "正在连接",
        "isReviewed": "True"
    },
    "Connecting. Please wait": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Соединение. Пожалуста подождите",
        "ZH": "正在连接。请稍候",
        "isReviewed": "True"
    },
    "Untitled": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Без названия",
        "ZH": "未命名",
        "isReviewed": "True"
    },
    "Add Subscription": {
        "source": [
            "Furious.Window.UserSubsWindow"
        ],
        "RU": "Добавить подписку",
        "ZH": "添加订阅",
        "isReviewed": "True"
    },
    "Enter subscription remark:": {
        "source": [
            "Furious.Window.UserSubsWindow"
        ],
        "RU": "Введите примечание о подписке:",
        "ZH": "输入订阅名称：",
        "isReviewed": "True"
    },
    "Enter subscription URL:": {
        "source": [
            "Furious.Window.UserSubsWindow"
        ],
        "RU": "Введите URL-адрес подписки:",
        "ZH": "输入订阅URL：",
        "isReviewed": "True"
    },
    "Edit Subscription": {
        "source": [
            "Furious.Window.UserSubsWindow"
        ],
        "RU": "Редактировать подписку",
        "ZH": "编辑订阅",
        "isReviewed": "True"
    },
    "Subscription List": {
        "source": [
            "Furious.Window.UserSubsWindow"
        ],
        "RU": "Список подписки",
        "ZH": "订阅列表",
        "isReviewed": "True"
    },
    "Add": {
        "source": [
            "Furious.Window.UserSubsWindow"
        ],
        "RU": "Добавить",
        "ZH": "添加",
        "isReviewed": "True"
    },
    "Unable to save log": {
        "source": [
            "Furious.Window.LogViewerWindow"
        ],
        "RU": "Не удалось сохранить журнал",
        "ZH": "无法保存日志",
        "isReviewed": "True"
    },
    "Text files (*.txt);;All files (*)": {
        "source": [
            "Furious.Window.LogViewerWindow"
        ],
        "RU": "Текстовые файлы (*.txt);;Все файлы (*)",
        "ZH": "文本文件 (*.txt);;所有文件 (*)",
        "isReviewed": "True"
    },
    "Error saving log": {
        "source": [
            "Furious.Window.LogViewerWindow"
        ],
        "RU": "Журнал сохранения ошибок",
        "ZH": "保存日志出错",
        "isReviewed": "True"
    },
    "Log Viewer": {
        "source": [
            "Furious.Window.LogViewerWindow"
        ],
        "RU": "Средство просмотра журналов",
        "ZH": "日志查看器",
        "isReviewed": "True"
    },
    "Asset file already exists. Overwrite?": {
        "source": [
            "Furious.Widget.XrayAssetViewerQListWidget"
        ],
        "RU": "Файл ресурсов уже существует. Перезаписать?",
        "ZH": "资源文件已存在。是否覆盖？",
        "isReviewed": "True"
    },
    "Error import asset file": {
        "source": [
            "Furious.Widget.XrayAssetViewerQListWidget"
        ],
        "RU": "Ошибка при импорте файла актива",
        "ZH": "导入资源文件出错",
        "isReviewed": "True"
    },
    "Import asset file success": {
        "source": [
            "Furious.Widget.XrayAssetViewerQListWidget"
        ],
        "RU": "Успешно импортирован файл ресурсов",
        "ZH": "导入资源文件成功",
        "isReviewed": "True"
    },
    "Xray-core Asset File": {
        "source": [
            "Furious.Window.XrayAssetViewerWindow"
        ],
        "RU": "Xray-core Asset File",
        "ZH": "Xray-core资源文件",
        "isReviewed": "True"
    },
    "Refresh": {
        "source": [
            "Furious.Window.XrayAssetViewerWindow"
        ],
        "RU": "Обновить",
        "ZH": "刷新",
        "isReviewed": "True"
    },
    "Open Asset Directory": {
        "source": [
            "Furious.Window.XrayAssetViewerWindow"
        ],
        "RU": "Откройте каталог ресурсов",
        "ZH": "打开资源目录",
        "isReviewed": "True"
    },
    "All files (*)": {
        "source": [
            "Furious.Window.XrayAssetViewerWindow"
        ],
        "RU": "Все файлы (*)",
        "ZH": "所有文件 (*)",
        "isReviewed": "True"
    },
    "Server": {
        "source": [
            "Furious.Widget.GuiHysteria1",
            "Furious.Widget.GuiHysteria2",
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Сервер",
        "ZH": "服务器",
        "isReviewed": "True"
    },
    "Log": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Журнал событий",
        "ZH": "日志",
        "isReviewed": "True"
    },
    "Show Furious Log": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Показать журнал Furious",
        "ZH": "显示Furious日志",
        "isReviewed": "True"
    },
    "Show Core Log": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Показать журнал ядра",
        "ZH": "显示内核日志",
        "isReviewed": "True"
    },
    "Show Tun2socks Log": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Показать журнал Tun2socks",
        "ZH": "显示Tun2socks日志",
        "isReviewed": "True"
    },
    "Update Subscription (Use Current Proxy)": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Обновление подписки (используйте текущий прокси-сервер)",
        "ZH": "更新订阅（使用当前代理）",
        "isReviewed": "True"
    },
    "Update Subscription (Force Proxy)": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Обновление подписки (обязательный прокси-сервер)",
        "ZH": "更新订阅（强制代理）",
        "isReviewed": "True"
    },
    "Update Subscription (No Proxy)": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Обновление подписки (не через прокси)",
        "ZH": "更新订阅（无代理）",
        "isReviewed": "True"
    },
    "Edit Subscription...": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Отредактируйте подписку...",
        "ZH": "编辑订阅...",
        "isReviewed": "True"
    },
    "Tools": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Иструменты",
        "ZH": "工具",
        "isReviewed": "True"
    },
    "Manage Xray-core Asset File...": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Управляйте файлом обьекта Xray-core...",
        "ZH": "管理Xray-core资源文件...",
        "isReviewed": "True"
    },
    "Check For Updates": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Проверка обновлений",
        "ZH": "检查更新",
        "isReviewed": "True"
    },
    "About": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "О нас",
        "ZH": "关于",
        "isReviewed": "True"
    },
    "Already started": {
        "source": [
            "Furious.Widget.Application"
        ],
        "RU": "Активированный",
        "ZH": "已启动",
        "isReviewed": "True"
    },
    "Furious Log": {
        "source": [
            "Furious.Widget.Application"
        ],
        "RU": "Furious журнал",
        "ZH": "Furious日志",
        "isReviewed": "True"
    },
    "Core Log": {
        "source": [
            "Furious.Widget.Application"
        ],
        "RU": "Лог ядра",
        "ZH": "内核日志",
        "isReviewed": "True"
    },
    "Tun2socks Log": {
        "source": [
            "Furious.Widget.Application"
        ],
        "RU": "Лог Tun2socks",
        "ZH": "Tun2socks日志",
        "isReviewed": "True"
    },
    "Connect": {
        "source": [
            "Furious.TrayActions.Connect"
        ],
        "RU": "Соединение",
        "ZH": "连接",
        "isReviewed": "True"
    },
    "Disconnect": {
        "source": [
            "Furious.TrayActions.Connect"
        ],
        "RU": "Рассоединение",
        "ZH": "断开连接",
        "isReviewed": "True"
    },
    "Unable to connect": {
        "source": [
            "Furious.Core.CoreManager",
            "Furious.TrayActions.Connect"
        ],
        "RU": "Не удается подключиться",
        "ZH": "无法连接",
        "isReviewed": "True"
    },
    "Server configuration empty. Please configure your server first": {
        "source": [
            "Furious.TrayActions.Connect"
        ],
        "RU": "Конфигурация сервера пуста. Пожалуйста, сначала настройте свой сервер",
        "ZH": "服务器配置为空。请先配置服务器",
        "isReviewed": "True"
    },
    "Connected": {
        "source": [
            "Furious.TrayActions.Connect"
        ],
        "RU": "Подключенный",
        "ZH": "已连接",
        "isReviewed": "True"
    },
    "Unknown error": {
        "source": [
            "Furious.TrayActions.Connect"
        ],
        "RU": "Неизвестная ошибка",
        "ZH": "未知错误",
        "isReviewed": "True"
    },
    "Failed to start core": {
        "source": [
            "Furious.TrayActions.Connect"
        ],
        "RU": "Сбой при запуске ядра",
        "ZH": "内核启动失败",
        "isReviewed": "True"
    },
    "Connection to server has been lost": {
        "source": [
            "Furious.TrayActions.Connect"
        ],
        "RU": "Соединение с сервером было потеряно",
        "ZH": "与服务器的连接已丢失",
        "isReviewed": "True"
    },
    "Core terminated unexpectedly": {
        "source": [
            "Furious.TrayActions.Connect"
        ],
        "RU": "Неожиданное завершение работы ядра",
        "ZH": "内核意外终止",
        "isReviewed": "True"
    },
    "Disconnected": {
        "source": [
            "Furious.TrayActions.Connect"
        ],
        "RU": "Разъединенный",
        "ZH": "已断开",
        "isReviewed": "True"
    },
    "Language": {
        "source": [
            "Furious.TrayActions.Language"
        ],
        "RU": "Языки",
        "ZH": "语言",
        "isReviewed": "True"
    },
    "Routing": {
        "source": [
            "Furious.TrayActions.Routing"
        ],
        "RU": "Маршрутизация",
        "ZH": "路由",
        "isReviewed": "True"
    },
    "VPN Mode": {
        "source": [
            "Furious.TrayActions.Settings"
        ],
        "RU": "VPN режим",
        "ZH": "VPN模式",
        "isReviewed": "True"
    },
    "VPN Mode Disabled (Administrator)": {
        "source": [
            "Furious.TrayActions.Settings"
        ],
        "RU": "VPN режим отключен (Administrator)",
        "ZH": "VPN模式已禁用（管理员）",
        "isReviewed": "True"
    },
    "VPN Mode Disabled (root)": {
        "source": [
            "Furious.TrayActions.Settings"
        ],
        "RU": "VPN режим отключен  (root)",
        "ZH": "VPN模式已禁用（root）",
        "isReviewed": "True"
    },
    "Settings": {
        "source": [
            "Furious.TrayActions.Settings"
        ],
        "RU": "Настройки",
        "ZH": "设置",
        "isReviewed": "True"
    },
    "Dark Mode": {
        "source": [
            "Furious.TrayActions.Settings"
        ],
        "RU": "Темный режим",
        "ZH": "深色模式",
        "isReviewed": "True"
    },
    "Startup On Boot": {
        "source": [
            "Furious.TrayActions.Settings"
        ],
        "RU": "Автозапуск",
        "ZH": "开机启动",
        "isReviewed": "True"
    },
    "Show Progress Bar When Connecting": {
        "source": [
            "Furious.TrayActions.Settings"
        ],
        "RU": "Отображение индикатора выполнения при подключении",
        "ZH": "连接时显示进度条",
        "isReviewed": "True"
    },
    "Show Tab And Spaces In Editor": {
        "source": [
            "Furious.TrayActions.Settings"
        ],
        "RU": "Показывать табуляцию и пробелы в редакторе",
        "ZH": "在编辑器中显示制表符和空格",
        "isReviewed": "True"
    },
    "System Proxy": {
        "source": [
            "Furious.TrayActions.SystemProxy"
        ],
        "RU": "Системный прокси",
        "ZH": "系统代理",
        "isReviewed": "True"
    },
    "Automatically Configure System Proxy": {
        "source": [
            "Furious.TrayActions.SystemProxy"
        ],
        "RU": "Автоматическая настройка системного прокси-сервера",
        "ZH": "自动配置系统代理",
        "isReviewed": "True"
    },
    "Do Not Change System Proxy": {
        "source": [
            "Furious.TrayActions.SystemProxy"
        ],
        "RU": "Не меняйте системный прокси-сервер",
        "ZH": "不修改系统代理",
        "isReviewed": "True"
    },
    "Furious is not able to run on this operating system": {
        "source": [
            "Furious.__main__"
        ],
        "RU": "Furious не может работать в этой операционной системе",
        "ZH": "Furious无法在此操作系统上运行",
        "isReviewed": "True"
    },
    "Operating system information": {
        "source": [
            "Furious.__main__"
        ],
        "RU": "Информация об операционной системе",
        "ZH": "操作系统信息",
        "isReviewed": "True"
    },
    "Furious encountered an internal error and needs to be stopped": {
        "source": [
            "Furious.__main__"
        ],
        "RU": "В Furious произошла внутренняя ошибка, и ее необходимо завершить",
        "ZH": "Furious遇到内部错误，需要终止",
        "isReviewed": "True"
    },
    "Furious stopped unexpectedly due to an unknown exception": {
        "source": [
            "Furious.__main__"
        ],
        "RU": "Программа Furious неожиданно завершилась из-за неизвестного исключения",
        "ZH": "由于一个未知异常，Furious意外终止",
        "isReviewed": "True"
    },
    "Crash log has been saved to": {
        "source": [
            "Furious.__main__"
        ],
        "RU": "Журнал сбоев был сохранен в",
        "ZH": "崩溃日志已保存到",
        "isReviewed": "True"
    },
    "Open crash log": {
        "source": [
            "Furious.__main__"
        ],
        "RU": "Открыть журнал ошибок",
        "ZH": "打开崩溃日志",
        "isReviewed": "True"
    },
    "Bypass Mainland China": {
        "source": [
            "Furious.TrayActions.Routing"
        ],
        "RU": "В обход Китая",
        "ZH": "绕过中国大陆",
        "isReviewed": "True"
    },
    "Global": {
        "source": [
            "Furious.TrayActions.Routing"
        ],
        "RU": "Весь трафик",
        "ZH": "全球",
        "isReviewed": "True"
    },
    "Custom": {
        "source": [
            "Furious.TrayActions.Routing"
        ],
        "RU": "Свои настройки",
        "ZH": "自定义",
        "isReviewed": "True"
    },
    "Furious cannot find any valid http proxy endpoint in the configuration": {
        "source": [
            "Furious.TrayActions.Connect"
        ],
        "RU": "Я не могу найти ни одной допустимой конечной точки http-прокси в конфигурации",
        "ZH": "Furious在配置中无法找到任何有效的http代理端点",
        "isReviewed": "True"
    },
    "Please complete your server configuration": {
        "source": [
            "Furious.TrayActions.Connect"
        ],
        "RU": "Пожалуйста, завершите настройку вашего сервера",
        "ZH": "请完善服务器配置",
        "isReviewed": "True"
    },
    "Help": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Помощь",
        "ZH": "帮助",
        "isReviewed": "True"
    },
    "Test Tcping Latency": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Задержка Tcping теста",
        "ZH": "测试Tcping延迟",
        "isReviewed": "True"
    },
    "Basic Configuration": {
        "source": [
            "Furious.Widget.GuiHysteria1",
            "Furious.Widget.GuiHysteria2",
            "Furious.Widget.GuiShadowsocks",
            "Furious.Widget.GuiTrojan",
            "Furious.Widget.GuiVLESS",
            "Furious.Widget.GuiVMess"
        ],
        "RU": "Базовая конфигурация",
        "ZH": "基本配置",
        "isReviewed": "True"
    },
    "Proxy": {
        "source": [
            "Furious.Widget.GuiHysteria1",
            "Furious.Widget.GuiHysteria2",
            "Furious.Widget.GuiShadowsocks",
            "Furious.Widget.GuiTrojan",
            "Furious.Widget.GuiVLESS",
            "Furious.Widget.GuiVMess"
        ],
        "RU": "Прокси",
        "ZH": "代理",
        "isReviewed": "True"
    },
    "Project Website": {
        "source": [
            "Furious.Widget.GuiHysteria1",
            "Furious.Widget.GuiHysteria2"
        ],
        "RU": "Веб-сайт проекта",
        "ZH": "项目网站",
        "isReviewed": "True"
    },
    "Other": {
        "source": [
            "Furious.Widget.GuiHysteria1",
            "Furious.Widget.GuiHysteria2"
        ],
        "RU": "Прочеи",
        "ZH": "其它",
        "isReviewed": "True"
    },
    "Encryption": {
        "source": [
            "Furious.Widget.GuiShadowsocks",
            "Furious.Widget.GuiVLESS"
        ],
        "RU": "Шифрование",
        "ZH": "加密",
        "isReviewed": "True"
    },
    "Password": {
        "source": [
            "Furious.Widget.GuiShadowsocks",
            "Furious.Widget.GuiTrojan"
        ],
        "RU": "Пароль",
        "ZH": "密码",
        "isReviewed": "True"
    },
    "Flow": {
        "source": [
            "Furious.Widget.GuiVLESS"
        ],
        "RU": "Поток",
        "ZH": "流控",
        "isReviewed": "True"
    },
    "Use Monochrome Tray Icon": {
        "source": [
            "Furious.TrayActions.Settings"
        ],
        "RU": "Использование монохромного значока в трее",
        "ZH": "使用单色托盘图标",
        "isReviewed": "True"
    },
    "Customize JSON Configuration...": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Настройте конфигурацию JSON...",
        "ZH": "自定义JSON配置...",
        "isReviewed": "True"
    },
    "Add VMess Server...": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Добавить сервер VMees...",
        "ZH": "添加VMess服务器...",
        "isReviewed": "True"
    },
    "Add VLESS Server...": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Добавить сервер VLESS...",
        "ZH": "添加VLESS服务器...",
        "isReviewed": "True"
    },
    "Add Shadowsocks Server...": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Добавить сервер Shadowsocks...",
        "ZH": "添加Shadowsocks服务器...",
        "isReviewed": "True"
    },
    "Add Trojan Server...": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Добавит сервер Trojan...",
        "ZH": "添加Trojan服务器...",
        "isReviewed": "True"
    },
    "Add Hysteria1 Server...": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Добавьте сервер Hysteria 1...",
        "ZH": "添加Hysteria1服务器...",
        "isReviewed": "True"
    },
    "Add Hysteria2 Server...": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Добавьте сервер Hysteria 2...",
        "ZH": "添加Hysteria2服务器...",
        "isReviewed": "True"
    },
    "Routing option with direct rules is not allowed in VPN mode": {
        "source": [
            "Furious.Core.CoreManager"
        ],
        "RU": "Опция маршрутизации с прямыми правилами не разрешена в режиме VPN",
        "ZH": "VPN模式下不允许使用直接规则的路由选项",
        "isReviewed": "True"
    },
    "Select and press Enter to activate configuration and connect": {
        "source": [
            "Furious.TrayActions.Connect"
        ],
        "RU": "Выберите и нажмите Enter, чтобы активировать конфигурацию и подключиться",
        "ZH": "选择并按下Enter键以激活配置并连接",
        "isReviewed": "True"
    }
}
